/*
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package org.exoplatform.services.jcr.impl.storage.value.fs;

import java.io.File;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.exoplatform.services.log.Log;
import org.exoplatform.services.jcr.impl.storage.value.cas.RecordAlreadyExistsException;
import org.exoplatform.services.jcr.impl.storage.value.cas.VCASException;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SAS.
 * 
 * CAS IO support covers some work will be produced in target FileIOChannels to make them
 * CASeable.<br/> - add value - delete value -
 * 
 * Date: 15.07.2008
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: CASableIOSupport.java 33195 2009-06-30 11:04:28Z dkatayev $
 */
public class CASableIOSupport {

  private static Log            LOG                  = ExoLogger.getLogger("jcr.CASeableIOSupport");

  public static final int       HASHFILE_ORDERNUMBER = 0;

  protected final FileIOChannel channel;

  protected final String        digestAlgo;

  CASableIOSupport(FileIOChannel channel, String digestAlgo) {
    this.channel = channel;
    this.digestAlgo = digestAlgo;
  }

  /**
   * Open digester output.<br/> Digester output will write into given file and calc hash for a
   * content.
   * 
   * @param file
   *          - destenation file
   * @return - digester output stream
   * @throws IOException
   */
  public FileDigestOutputStream openFile(File file) throws IOException {
    MessageDigest md;
    try {
      md = MessageDigest.getInstance(digestAlgo);
    } catch (NoSuchAlgorithmException e) {
      LOG.error("Can't wriet using " + digestAlgo + " algorithm, " + e, e);
      throw new IOException(e.getMessage());
    }

    return new FileDigestOutputStream(file, md);
  }

  /**
   * Construct file name of given hash.
   * 
   * @param hash String
   *          - digester hash
   */
  public File getFile(String hash) {
    // work with digest
    return new File(channel.rootDir, channel.makeFilePath(hash, 0));
  }
}
